	org 0x8000
	
	; set ay reg 7 = 0xf8 (all tone)
	ld de,0x07f8
	call set_ay_reg
	
main_loop
	; play the chord pointed to by chord_num
chord_num	ld hl,chord_seq_1
	ld l,(hl)	; hl now points to the chord

chord_phase	ld a,3
	inc a
	and 0x03
	ld (chord_phase+1),a
	or l
	ld l,a	; hl now points to the note within the chord
	ld l,(hl)	; read the note
	add hl,hl	; multiply by two (and strip out the 0x80 in h)
	ld d,0
	ld e,l
	call set_ay_reg
	ld e,h
	call set_ay_reg
	
	; count beat frames - six frames per beat
beat_phase	ld a,6
	dec a
	jr nz,no_new_beat

chan_a_note	ld hl,%0001000100100001	; a 1 means swap to the other chord
	add hl,hl
	jr nc,chan_a_note_oflow
	inc hl
	exx

	ld hl,chord_num+1
	ld a,0x01
	xor (hl)
	ld (hl),a

	ld l,0xff & tree_layer+1
	ld a, 0 + ( ((background_addr+1) & 0xff) xor ((foreground_addr+1) & 0xff) )
	xor (hl)
	ld (hl),a

	ld l,0xff & (tree_height + 1)
	ld a,0x04
	xor (hl)
	ld (hl),a

	ld l,0xff & (tree_width + 1)
	ld a,0x04
	xor (hl)
	ld (hl),a
	exx
	
chan_a_note_oflow
	ld (chan_a_note+1),hl

chan_a_rhythm	ld hl,%0101000100100101	; rhythm sequence for channel A
	add hl,hl
	jr nc,chan_a_rhy_oflow
	inc hl
	ld a,0x20	; set chan_a_vol to maximum volume
	ld (chan_a_vol+1),a
	; draw a tree
	exx	; need to preserve hl

tree_layer	ld hl,(foreground_addr+1)	; background_addr+1 for background
	ld de,0x0524	; begin tree 0x08 rows down, 0x24 characters across
		; 0x0824 for background
	add hl,de

tree_height
	ld b,12	; 8 for background
tree_lp	
	push hl
	ld a,b
	srl a
	add a,l
	ld l,a
tree_width
	ld a,13	; 9 for background
	sub b
tree_row_lp
	ld (hl),%00100101
	inc l
	dec a
	jr nz,tree_row_lp

	pop hl
	inc h
	djnz tree_lp
	
	ld b,6
	ld a,4	; 4 for background
	add a,l
	ld l,a
trunk_lp
	ld (hl),%00010000
	inc h
	djnz trunk_lp

	exx

chan_a_rhy_oflow
	ld (chan_a_rhythm+1),hl

	
pattern_pos	ld a,0
	inc a
	ld (pattern_pos+1),a
	and 0x1f
	jr nz,no_change_pattern
	
	ld hl,chord_num+1
	ld a,(hl)
	xor 0x02
	ld (hl),a
	
no_change_pattern
	ld a,6
no_new_beat
	ld (beat_phase+1),a
chan_a_vol	ld a,0x20
	cp 0x11	; decrement A if above 0x10
	ccf
	sbc a,0
	ld (chan_a_vol+1),a
	srl a
	ld e,a
	ld d,8
	call set_ay_reg
	
	halt
background_addr	ld de,background_data
foreground_addr	ld hl,foreground_data
	inc e	; move background on one byte (modulo 256)
	ld (background_addr+1),de
	inc l	; move foreground on two bytes (modulo 256)
	inc l
	ld (foreground_addr+1),hl
	
	ld ix,0x5800	; start of attributes
	ld c,0x18	; 0x18 rows
row_loop
	; store initial fg/bg addresses
	push de
	push hl
	ld a,%01101101	; clear background
	ld (de),a
	xor a
	ld (hl),a	; clear foreground
	inc l
	ld (hl),a
	; so now we're one byte away from 

	ld b,0x20	; 0x20 chars per row
char_loop
	ld a,(hl)	; read foreground
	inc l
	or a	; is foreground 0 (transparent)?
	jr nz,no_bg	; if not, don't read background
	ld a,(de)
no_bg
	inc e
	
	ld (ix),a
	inc ix
	djnz char_loop

	; TODO: can we eliminate the counter by checking screen address % 0x20 instead?

	pop hl	; recall initiall addresses
	pop de	; and move down one row
	inc h
	inc d

	dec c
	jr nz,row_loop
	
	jp main_loop
	
set_ay_reg
	ld bc,0xfffd
	out (c),d
	inc d
	ld b,0xbf
	out (c),e
	ret
	
	org 0x80f0
chord_seq_1	db chord_a & 0xff, chord_b & 0xff
chord_seq_2	db chord_c & 0xff, chord_c & 0xff

chord_a	db 0x00fc/2, 0x00ef/2, 0x00bd/2, 0x009f/2
chord_b	db 0x011c/2, 0x00fc/2, 0x00d6/2, 0x00a8/2
chord_c	db 0x013e/2, 0x00fc/2, 0x00d6/2, 0x00a8/2
;chords	db 0x01df/2, 0x017b/2, 0x013e/2, 0x00fc/2
	
background_data	equ 0xa000
foreground_data	equ 0xc000